#!/usr/bin/env python
# -*- coding: utf-8 -*-

from genignore.tests import test_helpers as ft
from os import path
from contextlib import contextmanager

inform = ft.log.info
shell = ft.shell
check_exists = ft.check_exists
log = ft.log


HOME = path.expanduser('~')
PWD = path.dirname(path.realpath(__file__))
ROOT = path.realpath(path.join(PWD, './../../'))
CACHE = path.join(HOME, '.genignore_cache')
LATEST = path.join(CACHE, 'latest.zip')
TEST_IGNORE = path.join(ROOT, 'test_ignore')


rm_ignore = lambda : shell('rm %s' % TEST_IGNORE)

def assert_exists(file_or_dir):
	assert check_exists(file_or_dir) == True, "FAIL: %s should exist" % file_or_dir


def do_cleanup():
	inform('setting up')
	inform('removing %s' % CACHE)

	shell('rm -rf %s/*.*' % CACHE)
	shell('rm -rf .venv')
	rm_ignore()

def set_venv_and_install_genignore():
	shell('virtualenv --no-site-packages .venv')
	shell('.venv/bin/python setup.py develop')


@contextmanager
def test_context(setup=None, teardown=None):
	do_cleanup()
	try:
		if callable(setup):
			setup()
		yield
	finally:
		try:
			if callable(teardown):
			teardown()
		except Exception, e:
			print(e)

		do_cleanup()


with test_context():

	shell('virtualenv --no-site-packages .venv')
	shell('.venv/bin/python setup.py develop')


	# TEST SYNC

	shell('.venv/bin/genignore sync', preserve_output=True)
	assert_exists(LATEST)
	inform('SUCCESS!')

	# TEST SYNC

	r = shell('.venv/bin/genignore list', preserve_output=True)
	inform(str(r[0]))
	inform('SUCCESS!')

	# TEST GEN

	shell('genignore gen osx --out=%s' % TEST_IGNORE, preserve_output=True)
	assert_exists(TEST_IGNORE)
	inform('SUCCESS!')
	rm_ignore()

	# TEST ADD

	shell('genignore gen osx --out=%s' % TEST_IGNORE, preserve_output=True)
	assert_exists(TEST_IGNORE)

	shell('genignore gen linux --out=%s --add' % TEST_IGNORE, preserve_output=True)
	assert_exists(TEST_IGNORE)
	inform('SUCCESS!')
	rm_ignore()

	# run bash -c "cat .test_ignore | grep genignore"
	# rm -f .test_ignore

	# res = shell('genignore gen osx jetbrains linux python | grep <genignore')
	# print(res)

	# deactivate
	# rm .test_ignore
	# # echo removing $HOME/.genignore_cache
	# # rm -rf $HOME/.genignore_cache/*.*

	# # echo removing .venv
	# # rm -rf .venv














#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
from os.path import join, realpath, abspath
import logging
import time
from datetime import datetime
from subprocess import Popen, PIPE


FORMAT = '[%(asctime)s][%(levelname)s] - %(message)s'
log = logging.getLogger('genignore.testing.log')
logging.basicConfig(format=FORMAT)
log.setLevel(logging.DEBUG)


realify = lambda *items: abspath(join(*items))


def check_exists(path, as_dir=False):
    if as_dir:
        log.debug('** checking if {0} dir exists'.format(path))
        return os.path.isdir(path)
    else:
        log.debug('** checking if {0} exists'.format(path))
        return os.path.exists(path)


def ensure_exists(path, as_dir=False):
    existent = check_exists(path, as_dir)

    if not existent:
        log.debug('** checking if {0} exists'.format(path))
        os.makedirs(path)


def touch(file_path):
    parts = file_path.split(os.sep)
    log.debug(parts)
    fpath, fname = os.sep.join(parts[:-1]), parts[-1]
    log.debug(fpath)
    log.debug(fname)
    ensure_exists(fpath, as_dir=True)
    return shell('touch %s' % file_path)


def shell(cmd, chdir=None, preserve_output=False):
    if isinstance(cmd, basestring):
        cmd = cmd.split(' ')

    if chdir:
        log.debug('changing dir into {0}'.format(chdir))
        os.chdir(chdir)

    log.debug('executing {0}'.format(" ".join(cmd)))
    p = Popen(cmd, stdout=PIPE, stderr=PIPE)

    out, err = [], []

    while p.poll() is None:
        try:
            _o = p.stdout.read()
            _e = p.stderr.read()

            log.debug(_o)
            if _e:
                log.error(_e)

            if preserve_output:
                out.append(_o.strip())
                err.append(_e.strip())
        except Exception, e:
            log.exception(e)
        time.sleep(0.125)

    if err:
        log.error(err)
    log.debug(out)
    if p.returncode != 0:
        log.debug('error executing {0}'.format(" ".join(cmd)))
    return p, "".join(out), "".join(err)


def ln(from_path, to_path, *args):
    return shell("ln -s {0} {1}".format(from_path, to_path))





from __future__ import absolute_import, print_function
import sys

from genignore.cli import parse
from genignore.core import GenController


def main(arguments):
    controller = GenController()
    return controller(parse(arguments))


def main_func():
    exit(main(sys.argv[1:]))


if __name__ == '__main__':
    main_func()



from __future__ import absolute_import, print_function
import six
import sys
import os
from os import path
from zipfile import ZipFile

from clint.textui import progress
from .helpers import print_notice
from .helpers import print_success
from .helpers import print_error
from .helpers import print_list
from .helpers import isatty

from . import updaters


MASTER_ARCHIVE = "https://codeload.github.com/github/gitignore/zip/refs/heads/main"
GENIGNORE_CACHE = ".genignore_cache"
LATEST_ZIP = "latest.zip"



class _CacheResolver(object):
    PROGRAM_CACHE = ".genignore_cache"
    LATEST_ZIP = "latest.zip"

    def get_cache_paths(self):
        folder = path.join(path.expanduser("~"), self.PROGRAM_CACHE)
        return folder, LATEST_ZIP

    def get_latest_file(self):
        folder, filename = self.get_cache_paths()
        return path.join(folder, filename)

    def get_templates_from_zipfile(self, latest_file):

        templates = dict()

        with ZipFile(latest_file, 'r') as latest:
            for zi in latest.infolist():
                name = path.split(zi.filename)[-1].split(".")[0].lower()
                if len(name):
                    templates[name] = zi.filename

        return templates

    def init_repo_templates(self, sync=None):

        folder, filename = self.get_cache_paths()
        latest_file = self.get_latest_file()
        dir_exists = path.isdir(folder)
        file_missing = not os.path.isfile(latest_file)

        if not dir_exists:
            print_notice("github template folder not found. Creating folder at %s" % folder)
            file_missing = True
            os.makedirs(folder)

        if file_missing or sync:
            update_latest_from_github(MASTER_ARCHIVE, latest_file)

        return self.get_templates_from_zipfile(latest_file)


class GenController(object):

    def __init__(self):
        self.resolver = _CacheResolver()

    def gen(self, args):
        given_names = [n.lower().strip() for n in args.names]

        templates = self.resolver.init_repo_templates()
        names = templates.keys()

        selected = dict()
        for n in given_names:
            if n in names:
                selected[n] = templates[n]
            else:
                print_error("No .gitignore template available for %s." % n)
                print_notice("You can view available templates by running `genignore list`")

                return 1

        if isatty():
            out = args.out
        else:
            out = sys.stdout
        build_gitignore(self.resolver, selected, out=out,
                        add_to_existing=args.add, original_templates=templates)
        return 0

    def sync(self, args):
        self.resolver.init_repo_templates(sync=True)
        return 0

    def list(self, args):
        templates = self.resolver.init_repo_templates()
        print_list(templates.keys(), title='Available Templates:')
        return 0

    def __call__(self, args):
        action = args.action
        if hasattr(self, action):
            return getattr(self, action)(args)
        else:
            raise NotImplementedError()

def update_latest_from_github(archive_uri, latest_file):

    print_notice("Fetching latest template(s) from %s to %s" % (archive_uri, latest_file))
    downloader = updaters.get('github')(archive_uri, latest_file)
    total_length = next(downloader)

    if total_length:
        with progress.Bar(label="Downloading ", expected_size=int(total_length)) as bar:
            for downloaded in downloader:
                bar.show(downloaded)

    print_success("Done!")




def build_gitignore(resolver, templates_for_merging, out=None, add_to_existing=False, original_templates={}):

    def make_separator(name, end=''):
        parts = [os.linesep, "# ", "<", end, "genignore ", name, ">", os.linesep]
        return "".join(parts)

    def extract_separator(line):
        res = line.replace("# <genignore ", "").replace(">", "")
        return res.strip()

    file_content = []
    existing_templates = []

    fname = path.join(*resolver.get_cache_paths())

    # find the file we want to load
    if out is sys.stdout or len(out) == 0:
        cwd = os.getcwd()
        gitignore_path = path.join(cwd, ".gitignore")
    else:
        gitignore_path = path.realpath(out)

    # collect existing genignore tags and orphan entries
    if os.path.isfile(gitignore_path):
        print_notice("file %s exists. Getting content..." % gitignore_path)
        with open(gitignore_path) as f:

            pushlines = True
            for line in f:
                if "# <genignore " in line:
                    existing_templates.append(extract_separator(line))
                    pushlines = False
                elif "# </genignore" in line:
                    pushlines = True
                else:
                    stripped_line = line.strip()
                    if pushlines and len(stripped_line):
                        file_content.append(stripped_line)

    if len(existing_templates):
        print_notice("file %s was generated by genignore. Found the following templates" % gitignore_path)
        print_list(existing_templates)
        if add_to_existing:
            print_notice("These will be added to the existing templates")
            for t in existing_templates:
                if t in original_templates:
                    templates_for_merging[t] = original_templates[t]
                else:
                    print_notice("Warning: template %s not found. Skipping..." % t)
        else:
            print_notice("Warning: will not add to existing file. Use --add to add these...")
            for e in existing_templates:
                if e in templates_for_merging:
                    del templates_for_merging[e]

    if len(templates_for_merging) == 0:
        print_notice("no templates remaining after filtering. Exiting")
        return 1

    print_notice("building .gitignore for (%s)" % ", ".join(templates_for_merging.keys()))

    with ZipFile(fname, 'r') as zipfile:
        for name, template in six.iteritems(templates_for_merging):
            with zipfile.open(template, 'r') as tmp:
                file_content.append(make_separator(name))
                tmp_content = tmp.read()
                if six.PY3:
                    tmp_content = tmp_content.decode('utf-8')
                file_content.append(tmp_content)
                file_content.append(make_separator(name, "/"))

    file_content.append(os.linesep)
    gitignore = os.linesep.join(file_content)

    if out is sys.stdout:
        out.write(gitignore)
        return 0

    with open(gitignore_path, 'w') as f:
        f.write(gitignore)
        print_success("Done writing .gitignore templates on file %s" % gitignore_path)



from os import path
import shutil
import unittest
import mock
import requests

# from genignore.genignore import parse
# from genignore.genignore import get_cache_paths
# from genignore.genignore import init_repo_templates
#
#
#
#
# def delete_cache():
# 	folder, _ = get_cache_paths()
# 	if path.isdir(folder):
# 		shutil.rmtree(folder)
#
#
# class TestGenignoreParse(unittest.TestCase):
#
# 	def test_assigns_names(self):
# 		args = parse(["gen", "osx", "python", "linux"])
# 		self.assertIsNotNone(args.names)
#
# 	def test_assigns_optionals_if_present(self):
# 		args = parse(["gen", "osx", "python", "linux", "--update"])
# 		self.assertTrue(args.update)
# 		self.assertEquals(args.action, "gen")
#
#
# class TestGenignore_init_repo_templates(unittest.TestCase):
#
# 	@mock.patch('requests.get')
# 	@mock.patch('genignore.genignore.get_templates_from_zipfile')
# 	def test_sync_if_folder_nonexistent(self, mocked_requests_get, mocked_zip):
# 		delete_cache()
# 		init_repo_templates()
# 		self.assertTrue(mocked_requests_get.called)
# 		self.assertTrue(mocked_zip.called)
#
# 	@mock.patch('requests.get')
# 	@mock.patch('genignore.genignore.get_templates_from_zipfile')
# 	def test_sync_if_folder_exists_but_sync_is_True(self, mocked_requests_get, mocked_zip):
# 		init_repo_templates(sync=True)
# 		self.assertTrue(mocked_requests_get.called)
# 		self.assertTrue(mocked_zip.called)


if __name__ == '__main__':
	unittest.main()
